#include <stdio.h>
#include <fcntl.h>
#include <r_asm.h>
#include <r_bin.h>

int main(int argc, char *argv[])
{
	struct r_bin_t bin;
	struct r_bin_symbol_t *symbols;
	struct r_bin_import_t *imports;
	struct r_asm_t a;
	struct r_asm_aop_t aop;
	int i, size, fd;
	char asm_code[1024];
	u64 baddr = 0, func_address = 0, func_offset = 0, func_size = 0,
		puts_address = 0;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s <bin file> <symbol name>\n", argv[0]);
		return 1;
	}

	printf( "Binary Injection (Demo 1)\n"
			"-------------------------\n");

	/* r_bin initialization */
	r_bin_init(&bin);
	/* Open file and autodetect format */
	r_bin_open(&bin, argv[1], 0, NULL);
	/* Get the base address */
	baddr = r_bin_get_baddr(&bin);
	/* Get the symbols & imports array */
	symbols = r_bin_get_symbols(&bin);
	imports = r_bin_get_imports(&bin);
	/* Get offsets addresses */
	for (i = 0; symbols && !symbols[i].last; i++)
		if (!strcmp(argv[2], symbols[i].name)) {
			func_offset = symbols[i].offset;
			func_address = baddr + symbols[i].rva;
			func_size = symbols[i].size;
		}
	for (i = 0; imports && !imports[i].last; i++)
		if (!strcmp("puts", imports[i].name)) {
			puts_address = baddr + imports[i].rva;
		}
	/* r_bin close */
	r_bin_close(&bin);
	free(symbols);
	free(imports);
	/* Print */
	printf( "%s offset: 0x%08llx\n"
			"%s address: 0x%08llx\n"
			"%s size: %lli\n"
			"puts address: 0x%08llx\n"
			"-------------------------\n",
			argv[2], func_offset, argv[2], func_address,
			argv[2], func_size, puts_address);

	/* r_asm initialization */
	r_asm_init(&a);
	/* Set an x86 asm plugin */
	r_asm_set(&a, "asm_x86_nasm");
	/* Assemble 64bits code */
	r_asm_set_bits(&a, 64);
	/* Set program counter = target function address */
	r_asm_set_pc(&a, func_address);
	sprintf(asm_code,
			"push rbp;"
			"mov rbp, rsp;"
			"mov rdi, _hello;"
			"call 0x%08llx;"
			"mov eax, 0x0;"
			"pop rbp;"
			"ret;"
			"_hello:.string HELLO WORLD!!!    ",
			puts_address);
	/* Assemble our code */
	size = r_asm_massemble(&a, &aop, asm_code);
	/* Print */
	printf( "Asm code: %s\n"
			"Hex: %s\n"
			"Code size: %i\n"
			"-------------------------\n",
			asm_code, aop.buf_hex, size);

	/* Overwrite the target function */
	if (size <= func_size) {
		fd = open(argv[1], O_RDWR);
		lseek(fd, func_offset, SEEK_SET);
		write(fd, &aop.buf, size);
		close(fd);
		printf("\n[Info] Code successfully injected\n\n");
	} else {
		printf("\n[Error] Size of injected code is bigger than function size\n\n");
		return 0;
	}
	
	return 0;
}

